﻿
using FLY.OBJComponents.Server;
using System.Collections.Generic;

namespace FLY.Weight.Server.Model
{
    /// <summary>
    /// 数据库 写操作
    /// </summary>
    public class HistoryDb
    {
        DbModel dbModel;
        public BulkDbSQLite<Lc_Flow, Db_Flow> FlowBuffer;
        public BulkDbMix<Db_Mix>[] MixBuffers;
        public BulkDbMix<Db_RollMix>[] RollMixBuffers;
        public BufferError ErrorBuffer;
        /// <summary>
        /// 
        /// </summary>
        /// <param name="dBModel"></param>
        public void Init(DbModel dBModel, int weighterCnt)
        {
            this.dbModel = dBModel;
            FlowBuffer = new BulkDbSQLite<Lc_Flow, Db_Flow>();
            FlowBuffer.Init(dBModel.TbFlow,
                (lc) => Lc_AutoMapperProfile.Mapper.Map<Db_Flow>(lc),
                (db) => Lc_AutoMapperProfile.Mapper.Map<Lc_Flow>(db)
                );

            MixBuffers = new BulkDbMix<Db_Mix>[weighterCnt];
            RollMixBuffers = new BulkDbMix<Db_RollMix>[weighterCnt];
            for (int i = 0; i < weighterCnt; i++)
            {
                MixBuffers[i] = new BulkDbMix<Db_Mix>();
                MixBuffers[i].Init(dBModel.TbMix, i,
                    (lc) => Lc_AutoMapperProfile.Mapper.Map<Lc_Mix, Db_Mix>(lc),
                    (db) => Lc_AutoMapperProfile.Mapper.Map<Db_Mix, Lc_Mix>(db)
                    );

                RollMixBuffers[i] = new BulkDbMix<Db_RollMix>();
                RollMixBuffers[i].Init(dBModel.TbRollMix, i,
                    (lc) => Lc_AutoMapperProfile.Mapper.Map<Lc_Mix, Db_RollMix>(lc),
                    (db) => Lc_AutoMapperProfile.Mapper.Map<Db_RollMix, Lc_Mix>(db)
                    );
            }
            ErrorBuffer = new OBJComponents.Server.BufferError();
            ErrorBuffer.Init(dBModel.TbError);
        }


        public void AddFlow(
            Lc_Flow flow
            )
        {
            FlowBuffer.Add(flow);
        }
        public void AddFlowRange(IEnumerable<Lc_Flow> flows)
        {
            FlowBuffer.AddRange(flows);
        }
        public void AddMix(int index,
            Lc_Mix mix
            )
        {
            MixBuffers[index].Add(mix);
        }

        public void AddRollMix(int index,
            Lc_Mix mix
            )
        {
            RollMixBuffers[index].Add(mix);
        }
    }
}
